@@ -17,7 +17,7 @@ require HTML::DOM::Node;
 require HTML::DOM::NodeList::Magic;
 
 our @ISA = qw'HTML::DOM::Node';
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 
 
 {
@@ -616,7 +616,7 @@ sub click { shift->trigger_event('click') }
 # used by innerHTML and insertAdjacentHTML
 sub _html_fragment_parser {
 		require HTML'DOM; # paranoia
-		(my $tb = new HTML::DOM::Element::HTML no_magic_forms => 1)
+		(my $tb = new HTML::DOM::Element::HTML:: no_magic_forms=>1)
 		  ->_set_ownerDocument(shift->ownerDocument);
 		$tb->parse(shift);
 		$tb->eof();
@@ -624,12 +624,17 @@ sub _html_fragment_parser {
 		$tb;
 }
 
+use constant _html_element_adds_newline =>
+ new HTML::Element 'foo' =>->as_HTML =~ /\n/;
+
 sub innerHTML {
 	my $self = shift;
 	my $old = join '', map $_->nodeType==ELEMENT_NODE
-			? substr(
-			   $_->as_HTML((undef)x2,{}),0,-1
-			  )
+			? _html_element_adds_newline
+			    ? substr(
+			       $_->as_HTML((undef)x2,{}),0,-1
+			      )
+			    : $_->as_HTML((undef)x2,{})
 			: encode_entities($_->data),$self->content_list
 	  if defined wantarray;
 	if(@_) {
@@ -776,14 +781,14 @@ L<HTML::DOM/CLASSES AND DOM INTERFACES>
 # ------- HTMLHeadElement interface ---------- #
 
 package HTML::DOM::Element::Head;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 sub profile { shift->_attr('profile' => @_) }
 
 # ------- HTMLLinkElement interface ---------- #
 
 package HTML::DOM::Element::Link;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 use Scalar::Util 'blessed';
 sub disabled {
@@ -911,7 +916,7 @@ sub _reset_style_sheet {
 # ------- HTMLTitleElement interface ---------- #
 
 package HTML::DOM::Element::Title;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 # This is what I call FWP (no lexical vars):
 sub text {
@@ -926,7 +931,7 @@ sub text {
 # ------- HTMLMetaElement interface ---------- #
 
 package HTML::DOM::Element::Meta;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 sub content   { shift->_attr('content'    => @_) }
 sub httpEquiv { shift->_attr('http-equiv' => @_) }
@@ -936,7 +941,7 @@ sub scheme    { shift->_attr('scheme'     => @_) }
 # ------- HTMLBaseElement interface ---------- #
 
 package HTML::DOM::Element::Base;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 *href =\& HTML::DOM::Element::Link::href;
 *target =\& HTML::DOM::Element::Link::target;
@@ -944,7 +949,7 @@ our @ISA = 'HTML::DOM::Element';
 # ------- HTMLIsIndexElement interface ---------- #
 
 package HTML::DOM::Element::IsIndex;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 sub form     { (shift->look_up(_tag => 'form'))[0] || () }
 # ~~~ Should this be the same as Select::form? I.e., should isindex ele-
@@ -954,7 +959,7 @@ sub prompt   { shift->_attr('prompt'  => @_) }
 # ------- HTMLStyleElement interface ---------- #
 
 package HTML::DOM::Element::Style;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 *disabled = \&HTML::DOM::Element::Link::disabled;
 *media =\& HTML::DOM::Element::Link::media;
@@ -974,7 +979,7 @@ sub sheet {
 # ------- HTMLBodyElement interface ---------- #
 
 package HTML::DOM::Element::Body;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 sub aLink      { shift->_attr( aLink      => @_) }
 sub background { shift->_attr( background => @_) }
@@ -999,7 +1004,7 @@ sub event_handler {
 # ------- HTMLUListElement interface ---------- #
 
 package HTML::DOM::Element::UL;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 sub compact { shift->_attr( compact => @_ ? $_[0]?'compact': undef : () ) }
 sub type { lc shift->_attr( type => @_) }
@@ -1007,7 +1012,7 @@ sub type { lc shift->_attr( type => @_) }
 # ------- HTMLOListElement interface ---------- #
 
 package HTML::DOM::Element::OL;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 sub start { shift->_attr( start => @_) }
 *compact=\&HTML::DOM::Element::UL::compact;
@@ -1016,28 +1021,28 @@ sub start { shift->_attr( start => @_) }
 # ------- HTMLDListElement interface ---------- #
 
 package HTML::DOM::Element::DL;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 *compact=\&HTML::DOM::Element::UL::compact;
 
 # ------- HTMLDirectoryElement interface ---------- #
 
 package HTML::DOM::Element::Dir;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 *compact=\&HTML::DOM::Element::UL::compact;
 
 # ------- HTMLMenuElement interface ---------- #
 
 package HTML::DOM::Element::Menu;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 *compact=\&HTML::DOM::Element::UL::compact;
 
 # ------- HTMLLIElement interface ---------- #
 
 package HTML::DOM::Element::LI;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 *type =\& HTML::DOM::Element::Link::type;
 sub value { shift->_attr( value => @_) }
@@ -1045,49 +1050,49 @@ sub value { shift->_attr( value => @_) }
 # ------- HTMLDivElement interface ---------- #
 
 package HTML::DOM::Element::Div;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 sub align { lc shift->_attr( align => @_) }
 
 # ------- HTMLParagraphElement interface ---------- #
 
 package HTML::DOM::Element::P;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 *align =\& HTML::DOM::Element::Div::align;
 
 # ------- HTMLHeadingElement interface ---------- #
 
 package HTML::DOM::Element::Heading;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 *align =\& HTML::DOM::Element::Div::align;
 
 # ------- HTMLQuoteElement interface ---------- #
 
 package HTML::DOM::Element::Quote;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 sub cite { shift->_attr( cite => @_) }
 
 # ------- HTMLPreElement interface ---------- #
 
 package HTML::DOM::Element::Pre;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 sub width { shift->_attr( width => @_) }
 
 # ------- HTMLBRElement interface ---------- #
 
 package HTML::DOM::Element::Br;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 sub clear { lc shift->_attr( clear => @_) }
 
 # ------- HTMLBaseFontElement interface ---------- #
 
 package HTML::DOM::Element::BaseFont;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 sub color { shift->_attr( color => @_) }
 sub face  { shift->_attr( face  => @_) }
@@ -1096,7 +1101,7 @@ sub size  { shift->_attr( size  => @_) }
 # ------- HTMLBaseFontElement interface ---------- #
 
 package HTML::DOM::Element::Font;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 *color =\& HTML::DOM::Element::BaseFont::color;
 *face =\& HTML::DOM::Element::BaseFont::face;
@@ -1105,7 +1110,7 @@ our @ISA = 'HTML::DOM::Element';
 # ------- HTMLHRElement interface ---------- #
 
 package HTML::DOM::Element::HR;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 *align =\& HTML::DOM::Element::Div::align;
 sub noShade { shift->_attr( noshade => @_ ? $_[0]?'noshade':undef : () ) }
@@ -1116,7 +1121,7 @@ sub noShade { shift->_attr( noshade => @_ ? $_[0]?'noshade':undef : () ) }
 # ------- HTMLModElement interface ---------- #
 
 package HTML::DOM::Element::Mod;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 *cite =\& HTML::DOM::Element::Quote::cite;
 sub dateTime  { shift->_attr( datetime  => @_) }
@@ -1124,7 +1129,7 @@ sub dateTime  { shift->_attr( datetime  => @_) }
 # ------- HTMLAnchorElement interface ---------- #
 
 package HTML::DOM::Element::A;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 sub accessKey  { shift->_attr(               accesskey  => @_) }
 *   charset    =\&HTML::DOM::Element::Link::charset           ;
@@ -1262,7 +1267,7 @@ sub search {
 # ------- HTMLImageElement interface ---------- #
 
 package HTML::DOM::Element::Img;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 sub lowSrc  { shift->attr(               lowsrc  => @_) }
 *   name  = \&HTML::DOM::Element::Meta::name            ;
@@ -1281,7 +1286,7 @@ sub vspace   { shift->_attr(              vspace   => @_) }
 # ------- HTMLObjectElement interface ---------- #
 
 package HTML::DOM::Element::Object;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 *form=\&HTML::DOM::Element::Select::form;
 sub code  { shift->_attr(               code  => @_) }
@@ -1306,7 +1311,7 @@ sub contentDocument{}
 # ------- HTMLParamElement interface ---------- #
 
 package HTML::DOM::Element::Param;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 *name=\&HTML::DOM::Element::Meta::name;
 *type=\&HTML::DOM::Element::Link::type;
@@ -1316,7 +1321,7 @@ sub valueType{lc shift->_attr(valuetype=>@_)}
 # ------- HTMLAppletElement interface ---------- #
 
 package HTML::DOM::Element::Applet;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 * align    = \ & HTML::DOM::Element::Div::align       ;
 * alt      = \ & HTML::DOM::Element::Img::alt         ;
@@ -1333,7 +1338,7 @@ sub object { shift -> _attr ( object => @_ ) }
 # ------- HTMLMapElement interface ---------- #
 
 package HTML::DOM::Element::Map;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 sub areas { # ~~~ I need to make this cache the resulting collection obj
 	my $self = shift;
@@ -1354,7 +1359,7 @@ sub areas { # ~~~ I need to make this cache the resulting collection obj
 # ------- HTMLAreaElement interface ---------- #
 
 package HTML::DOM::Element::Area;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 * alt       = \ & HTML::DOM::Element::Img::alt         ;
 sub coords { shift -> _attr ( coords => @_ ) }
@@ -1371,7 +1376,7 @@ sub noHref { shift->attr ( nohref => @_ ? $_[0] ? 'nohref' : undef : () ) }
 # ------- HTMLScriptElement interface ---------- #
 
 package HTML::DOM::Element::Script;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 * text    = \ &HTML::DOM::Element::Title::text   ;
 sub htmlFor { shift -> _attr ( for   => @_ )      }
@@ -1384,7 +1389,7 @@ sub defer { shift -> _attr ( defer => @_ ? $_[0] ? 'defer' : undef : () ) }
 # ------- HTMLFrameSetElement interface ---------- #
 
 package HTML::DOM::Element::FrameSet;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 sub rows { shift -> _attr ( rows   => @_ )      }
 sub cols   { shift -> _attr ( cols => @_ )      }
@@ -1392,7 +1397,7 @@ sub cols   { shift -> _attr ( cols => @_ )      }
 # ------- HTMLFrameElement interface ---------- #
 
 package HTML::DOM::Element::Frame;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 sub frameBorder { lc shift -> _attr ( frameBorder  => @_ )      }
 sub longDesc    { shift -> _attr ( longdesc     => @_ )      }
@@ -1412,7 +1417,7 @@ sub contentWindow {
 # ------- HTMLIFrameElement interface ---------- #
 
 package HTML::DOM::Element::IFrame;
-our $VERSION = '0.042';
+our $VERSION = '0.043';
 our @ISA = 'HTML::DOM::Element';
 *align  = \&HTML::DOM::Element::Div::align;
 *frameBorder = \&HTML::DOM::Element::Frame::frameBorder;
